@@ -13,7 +13,7 @@ use XML::LibXML::XPathContext;
 use XML::LibXML;
 use URI;
 
-our $VERSION = "0.0136";
+our $VERSION = "0.0137";
 
 with "Plack::Auth::SSO";
 
@@ -71,16 +71,27 @@ sub to_app {
         state $cas = Authen::CAS::Client->new($self->cas_url());
 
         my $env = $_[0];
+        my $log = $self->log();
 
         my $request = Plack::Request->new($env);
         my $session = Plack::Session->new($env);
         my $params  = $request->query_parameters();
 
+        if( $log->is_debug() ){
+
+            $log->debugf( "incoming query parameters: %s", [$params->flatten] );
+            $log->debugf( "session: %s", $session->dump() );
+            $log->debugf( "session_key for auth_sso: %s" . $self->session_key() );
+
+        }
+
         my $auth_sso = $self->get_auth_sso($session);
 
         #already got here before
         if (is_hash_ref($auth_sso)) {
 
+            $log->debug( "auth_sso already present" );
+
             return $self->redirect_to_authorization();
 
         }
@@ -100,6 +111,8 @@ sub to_app {
 
         if ( is_string($ticket) && $self->csrf_token_valid($session,$state) ) {
 
+            $log->debug( "ticket present and csrf token valid: entering callback phase" );
+
             $service_uri->query_form( state => $state );
             my $r = $cas->service_validate($service_uri->as_string(), $ticket);
 
@@ -107,6 +120,8 @@ sub to_app {
 
             if ($r->is_success) {
 
+                $log->debug( "ticket validation successfull" );
+
                 my $doc = $r->doc();
 
                 $self->set_auth_sso(
@@ -130,6 +145,8 @@ sub to_app {
             #e.g. "Can't connect to localhost:8443 (certificate verify failed)"
             elsif( $r->is_error() ) {
 
+                $log->error( "ticket validation failed: unexpected error" );
+
                 $self->set_auth_sso_error( $session, {
                     package    => __PACKAGE__,
                     package_id => $self->id,
@@ -142,6 +159,8 @@ sub to_app {
             #$r->is_failure() -> authenticationFailure: return to authentication url
             else {
 
+                $log->error( "ticket validation failed: authentication failure" );
+
                 my $failure = $self->parse_failure( $r->doc );
 
                 if ( $failure->{type} ne "INVALID_TICKET" ) {
@@ -159,6 +178,8 @@ sub to_app {
 
             $self->cleanup( $session );
 
+            $log->error( "ticket given, but csrf token not present" );
+
             $self->set_auth_sso_error( $session,{
                 package    => __PACKAGE__,
                 package_id => $self->id,
@@ -182,6 +203,8 @@ sub to_app {
         #no ticket or ticket validation failed
         my $login_url = $cas->login_url( $service_uri->as_string() )->as_string;
 
+        $log->info( "redirecting to cas login $login_url" );
+
         [302, [Location => $login_url], []];
 
     };
@@ -268,6 +291,12 @@ base url of the CAS service
 
 =back
 
+=head1 LOGGING
+
+All subclasses of L<Plack::Auth::SSO> use L<Log::Any>
+to log messages to the category that equals the current
+package name.
+
 =head1 ERRORS
 
 Cf. L<https://apereo.github.io/cas/4.2.x/protocol/CAS-Protocol-Specification.html#253-error-codes>
