@@ -13,7 +13,7 @@ use WWW::ORCID;
 use JSON;
 use Plack::Auth::SSO::ResponseParser::ORCID;
 
-our $VERSION = "0.0136";
+our $VERSION = "0.0137";
 
 with "Plack::Auth::SSO";
 
@@ -109,6 +109,10 @@ sub redirect_to_login {
         redirect_uri => $redirect_uri->as_string()
     );
 
+    $self->log()->info(
+        "redirecting to orcid login $auth_uri"
+    );
+
     [ 302, [ Location => $auth_uri ], [] ];
 
 }
@@ -130,9 +134,19 @@ sub to_app {
 
         my $auth_sso = $self->get_auth_sso($session);
 
+        if( $self->log()->is_debug() ){
+
+            $self->log()->debugf( "incoming query parameters: %s", [$params->flatten] );
+            $self->log()->debugf( "session: %s", $session->dump() );
+            $self->log()->debugf( "session_key for auth_sso: %s ", $self->session_key() );
+
+        }
+
         #already got here before
         if ( is_hash_ref($auth_sso) ) {
 
+            $self->log()->debug( "auth_sso already present" );
+
             return $self->redirect_to_authorization();
 
         }
@@ -143,6 +157,10 @@ sub to_app {
         #callback phase
         if ( is_string($code) && $self->csrf_token_valid($session,$state) ) {
 
+            $self->log()->debug(
+                "parameter code and state present: entering callback phase"
+            );
+
             $self->cleanup( $session );
 
             my $error             = $params->get("error");
@@ -177,6 +195,12 @@ sub to_app {
 
                     my $orcid_res = $json->decode( $body );
 
+                    $self->log()->errorf(
+                        "unable to retrieve access token. Error: %s, Error description: %s",
+                        $orcid_res->{error},
+                        $orcid_res->{error_description}
+                    ) if $self->log()->is_error();
+
                     return $self->redirect_to_login( $request, $session ) if $orcid_res->{error} eq "401";
 
                     $self->set_auth_sso_error( $session, {
@@ -189,6 +213,10 @@ sub to_app {
                 }
                 else {
 
+                    $self->log()->errorf(
+                        "unable to retrieve access token. See full body in auth_sso_error."
+                    ) if $self->log()->is_error();
+
                     $self->set_auth_sso_error( $session, {
                         package    => __PACKAGE__,
                         package_id => $self->id,
@@ -226,6 +254,8 @@ sub to_app {
 
         elsif( is_string($code) ) {
 
+            $self->log()->error( "parameter code given, but csrf token not present or invalid" );
+
             $self->cleanup( $session );
 
             $self->set_auth_sso_error( $session,{
@@ -241,6 +271,10 @@ sub to_app {
         #request phase
         else {
 
+            $self->log()->debug(
+                "neither code nor state given: entering request phase"
+            );
+
             $self->cleanup( $session );
 
             $self->redirect_to_login( $request, $session );
@@ -428,6 +462,12 @@ Please consult the ORCID to make sure that the parameters "public" and "scope" d
 
 =back
 
+=head1 LOGGING
+
+All subclasses of L<Plack::Auth::SSO> use L<Log::Any>
+to log messages to the category that equals the current
+package name.
+
 =head1 ERRORS
 
 Known ORCID errors are stored in the session key auth_sso_error ( L<Plack::Auth::SSO> ).
